﻿using ProjectMonorail.Entities;

namespace ProjectMonorail.DrawingObjects
{
    /// <summary>
    /// Класс, отвечающий за прорисовку и перемещение объекта-сущности
    /// </summary>
    public class DrawingExtendedMonorail : DrawingMonorail
    {
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="mainColor">Основной цвет</param>
        /// <param name="additionalColor">Дополнительный цвет</param>
        /// <param name="magneticRail">Признак наличия магнитной рельсы</param>
        /// <param name="extraCabin">Признак наличия дополнительной кабины</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        public DrawingExtendedMonorail(int speed, double weight, Color mainColor, Color additionalColor, bool magneticRail,
            bool extraCabin, int width, int height) : base(speed, weight, mainColor, width, height, 186, 92)
        {
            if (!magneticRail && !extraCabin)
            {
                _monorailWidth = 117;
                _monorailHeight = 56;
            }
            if (!magneticRail && extraCabin)
            {
                _monorailWidth = 183;
                _monorailHeight = 56;
            }
            if (EntityMonorail != null)
            {
                EntityMonorail = new EntityExtendedMonorail(speed, weight, mainColor,
                additionalColor, magneticRail, extraCabin);
            }
        }

        public override void DrawTransport(Graphics g)
        {
            if (EntityMonorail is not EntityExtendedMonorail extendedMonorail)
            {
                return;
            }
            Pen mainPen = new Pen(Color.Black, 2);
            Pen additionalPen = new(Color.Blue);
            Brush additionalBrush = new SolidBrush(extendedMonorail.AdditionalColor);
            Brush brBlue = new SolidBrush(Color.Blue);
            Brush brBlack = new SolidBrush(Color.Black);
            Brush brWhite = new SolidBrush(Color.White);
            Brush brGray = new SolidBrush(Color.Gray);

            base.DrawTransport(g);

            //магнитная рельса
            if (extendedMonorail.MagneticRail)
            {
                g.DrawRectangle(mainPen, _startPosX + 2, _startPosY + 58, 184, 18);
                g.FillRectangle(brGray, _startPosX + 2, _startPosY + 58, 184, 18);
                for (int i = 0; i < 4; i++)
                {
                    g.DrawRectangle(mainPen, _startPosX + 35 + 35 * i, _startPosY + 77, 8, 15);
                    g.FillRectangle(brGray, _startPosX + 35 + 35 * i, _startPosY + 77, 8, 15);
                }
            }

            //дополнительная кабина
            if (extendedMonorail.ExtraCabin)
            {
                //корпус дополнительной кабины
                g.FillRectangle(additionalBrush, _startPosX + 118, _startPosY + 15, 65, 31);
                g.DrawRectangle(mainPen, _startPosX + 118, _startPosY + 15, 65, 31);
                g.DrawLine(additionalPen, _startPosX + 118, _startPosY + 31, _startPosX + 183, _startPosY + 31);

                //дверь дополнительной кабины
                g.FillRectangle(brBlue, _startPosX + 146, _startPosY + 21, 7, 20);
                g.DrawRectangle(mainPen, _startPosX + 146, _startPosY + 21, 7, 20);

                //окна дополнительной кабины
                g.FillRectangle(brBlue, _startPosX + 130, _startPosY + 18, 6, 9);
                g.DrawRectangle(mainPen, _startPosX + 130, _startPosY + 18, 6, 9);
                g.FillRectangle(brBlue, _startPosX + 169, _startPosY + 18, 6, 9);
                g.DrawRectangle(mainPen, _startPosX + 169, _startPosY + 18, 6, 9);

                //колеса и тележка дополнительной кабины
                g.FillRectangle(brBlack, _startPosX + 126, _startPosY + 47, 15, 6);
                g.DrawRectangle(mainPen, _startPosX + 126, _startPosY + 47, 15, 6);
                g.FillRectangle(brBlack, _startPosX + 159, _startPosY + 47, 15, 6);
                g.DrawRectangle(mainPen, _startPosX + 159, _startPosY + 47, 15, 6);
                g.FillEllipse(brWhite, _startPosX + 128, _startPosY + 47, 10, 9);
                g.DrawEllipse(mainPen, _startPosX + 128, _startPosY + 47, 10, 9);
                g.FillEllipse(brWhite, _startPosX + 161, _startPosY + 47, 10, 9);
                g.DrawEllipse(mainPen, _startPosX + 161, _startPosY + 47, 10, 9);
            }
        }
    }
}
