﻿using ProjectMonorail.DrawingObjects;
using ProjectMonorail.Entities;

namespace ProjectMonorail
{
    /// <summary>
    /// Расширение для класса EntityMonorail
    /// </summary>
    public static class ExtentionDrawingMonorail
    {
        /// <summary>
        /// Создание объекта из строки
        /// </summary>
        /// <param name="info">Строка с данными для создания объекта</param>
        /// <param name="separatorForObject">Разделитель даннных</param>
        /// <param name="width">Ширина</param>
        /// <param name="height">Высота</param>
        /// <returns>Объект</returns>
        public static DrawingMonorail? CreateDrawingMonorail(this string info, char separatorForObject, int width, int height)
        {
            string[] strs = info.Split(separatorForObject);
            if (strs.Length == 3)
            {
                return new DrawingMonorail(Convert.ToInt32(strs[0]),
                Convert.ToInt32(strs[1]), Color.FromName(strs[2]), width, height);
            }
            if (strs.Length == 6)
            {
                return new DrawingExtendedMonorail(Convert.ToInt32(strs[0]),
                Convert.ToInt32(strs[1]),
                Color.FromName(strs[2]),
                Color.FromName(strs[3]),
                Convert.ToBoolean(strs[4]),
                Convert.ToBoolean(strs[5]),
                width, height);
            }
            return null;
        }

        /// <summary>
        /// Получение данных для сохранения в файл
        /// </summary>
        /// <param name="drawingMonorail">Сохраняемый объект</param>
        /// <param name="separatorForObject">Разделитель даннных</param>
        /// <returns>Строка с данными по объекту</returns>
        public static string GetDataForSave(this DrawingMonorail drawingMonorail, char separatorForObject)
        {
            var monorail = drawingMonorail.EntityMonorail;
            if (monorail == null)
            {
                return string.Empty;
            }
            var str = $"{monorail.Speed}{separatorForObject}{monorail.Weight}{separatorForObject}{monorail.MainColor.Name}";
            if (monorail is not EntityExtendedMonorail extendedMonorail)
            {
                return str;
            }
            return $"{str}{separatorForObject}{extendedMonorail.AdditionalColor.Name}{separatorForObject}{extendedMonorail.MagneticRail}{separatorForObject}{extendedMonorail.ExtraCabin}";
        }
    }
}