﻿using ProjectMonorail.DrawingObjects;
using ProjectMonorail.MovementStrategy;

namespace ProjectMonorail
{
    /// <summary>
    /// Форма работы с объектом "Монорельс"
    /// </summary>
    public partial class FormMonorail : Form
    {
        /// <summary>
        /// Поле-объект для прорисовки объекта
        /// </summary>
        private DrawingMonorail? _drawingMonorail;

        /// <summary>
        /// Стратегия перемещения
        /// </summary>
        private AbstractStrategy? _strategy;

        /// <summary>
        /// Выбранный монорельс
        /// </summary>
        public DrawingMonorail? SelectedMonorail { get; private set; }

        /// <summary>
        /// Инициализация формы
        /// </summary>
        public FormMonorail()
        {
            InitializeComponent();
            _strategy = null;
            SelectedMonorail = null;
        }

        /// <summary>
        /// Метод прорисовки транспорта
        /// </summary>
        private void Draw()
        {
            if (_drawingMonorail == null)
            {
                return;
            }
            Bitmap bmp = new(pictureBoxMonorail.Width, pictureBoxMonorail.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _drawingMonorail.DrawTransport(gr);
            pictureBoxMonorail.Image = bmp;
        }

        /// <summary>
        /// Обработка нажатия кнопки "Создать расширенный монорельс"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonCreateExtendedMonorail_Click(object sender, EventArgs e)
        {
            Random random = new();
            Color color = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                color = dialog.Color;
            }

            Color additionalColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            ColorDialog additionalDialog = new();
            if (additionalDialog.ShowDialog() == DialogResult.OK)
            {
                additionalColor = additionalDialog.Color;
            }

            _drawingMonorail = new DrawingExtendedMonorail(random.Next(200, 400), random.Next(1000, 3000), color, additionalColor,
             Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)), pictureBoxMonorail.Width, pictureBoxMonorail.Height);
            _drawingMonorail.SetPosition(random.Next(10, 100), random.Next(10, 100));
            Draw();
        }

        /// <summary>
        /// Обработка нажатия кнопки "Создать монорельс"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonCreateMonorail_Click(object sender, EventArgs e)
        {
            Random random = new();
            Color color = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                color = dialog.Color;
            }
            _drawingMonorail = new DrawingMonorail(random.Next(200, 400), random.Next(1000, 3000), color, pictureBoxMonorail.Width, pictureBoxMonorail.Height);
            _drawingMonorail.SetPosition(random.Next(10, 100), random.Next(10, 100));
            Draw();
        }

        /// <summary>
        /// Изменение положения монорельса
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonMove_Click(object sender, EventArgs e)
        {
            if (_drawingMonorail == null)
            {
                return;
            }
            string name = ((Button)sender)?.Name ?? string.Empty;
            switch (name)
            {
                case "buttonUp":
                    _drawingMonorail.MoveTransport(DirectionType.Up);
                    break;
                case "buttonDown":
                    _drawingMonorail.MoveTransport(DirectionType.Down);
                    break;
                case "buttonLeft":
                    _drawingMonorail.MoveTransport(DirectionType.Left);
                    break;
                case "buttonRight":
                    _drawingMonorail.MoveTransport(DirectionType.Right);
                    break;
            }
            Draw();
        }

        /// <summary>
        /// Обработка нажатия кнопки "Шаг"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonStep_Click(object sender, EventArgs e)
        {
            if (_drawingMonorail == null)
            {
                return;
            }
            if (comboBoxStrategy.Enabled)
            {
                _strategy = comboBoxStrategy.SelectedIndex
                switch
                {
                    0 => new MoveToCenter(),
                    1 => new MoveToBorder(),
                    _ => null,
                };
                if (_strategy == null)
                {
                    return;
                }
                _strategy.SetData(new DrawingObjectMonorail(_drawingMonorail), pictureBoxMonorail.Width, pictureBoxMonorail.Height);
            }
            if (_strategy == null)
            {
                return;
            }
            comboBoxStrategy.Enabled = false;
            _strategy.MakeStep();
            Draw();
            if (_strategy.GetStatus() == Status.Finish)
            {
                comboBoxStrategy.Enabled = true;
                _strategy = null;
            }
        }

        /// <summary>
        /// Выбор монорельса
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonSelectMonorail_Click(object sender, EventArgs e)
        {
            SelectedMonorail = _drawingMonorail;
            DialogResult = DialogResult.OK;
        }
    }
}