﻿using ProjectMonorail.DrawingObjects;
using ProjectMonorail.Entities;

namespace ProjectMonorail
{
    public partial class FormMonorailConfig : Form
    {
        /// <summary>
        /// Переменная-выбранный монорельс
        /// </summary>
        DrawingMonorail? _monorail = null;

        /// <summary>
        /// Событие
        /// </summary>
        private event Action<DrawingMonorail>? EventAddMonorail;

        /// <summary>
        /// Конструктор
        /// </summary>
        public FormMonorailConfig()
        {
            InitializeComponent();
            panelBlack.MouseDown += panelColor_MouseDown;
            panelPurple.MouseDown += panelColor_MouseDown;
            panelGray.MouseDown += panelColor_MouseDown;
            panelGreen.MouseDown += panelColor_MouseDown;
            panelRed.MouseDown += panelColor_MouseDown;
            panelWhite.MouseDown += panelColor_MouseDown;
            panelYellow.MouseDown += panelColor_MouseDown;
            panelBlue.MouseDown += panelColor_MouseDown;

            buttonCancel.Click += (s, e) => Close();
        }

        /// Добавление события
        /// </summary>
        /// <param name="ev">Привязанный метод</param>
        public void AddEvent(Action<DrawingMonorail> ev)
        {
            if (EventAddMonorail == null)
            {
                EventAddMonorail = ev;
            }
            else
            {
                EventAddMonorail += ev;
            }
        }

        /// <summary>
        /// Отрисовать монорельс
        /// </summary>
        private void DrawMonorail()
        {
            Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _monorail?.SetPosition(5, 5);
            _monorail?.DrawTransport(gr);
            pictureBoxObject.Image = bmp;
        }

        /// <summary>
        /// Передаем информацию при нажатии на Label
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelObject_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Label)?.DoDragDrop((sender as Label)?.Name,
           DragDropEffects.Move | DragDropEffects.Copy);
        }

        private void panelColor_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Panel)?.DoDragDrop((sender as Panel)?.BackColor,
           DragDropEffects.Move | DragDropEffects.Copy);
        }

        /// <summary>
        /// Проверка получаемой информации (ее типа на соответствие требуемому)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data?.GetDataPresent(DataFormats.Text) ?? false)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void labelColor_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data?.GetDataPresent(typeof(Color)) ?? false)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        /// <summary>
        /// Действия при приеме перетаскиваемой информации
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragDrop(object sender, DragEventArgs e)
        {
            switch (e.Data?.GetData(DataFormats.Text).ToString())
            {
                case "labelSimpleObject":
                    _monorail = new DrawingMonorail((int)numericUpDownSpeed.Value,
                   (int)numericUpDownWeight.Value, Color.White, pictureBoxObject.Width,
                   pictureBoxObject.Height);
                    break;
                case "labelModifiedObject":
                    _monorail = new DrawingExtendedMonorail((int)numericUpDownSpeed.Value,
                   (int)numericUpDownWeight.Value, Color.White, Color.Black, checkBoxMagneticRail.Checked,
                   checkBoxExtraCabin.Checked, pictureBoxObject.Width, pictureBoxObject.Height);
                    break;
            }
            DrawMonorail();
        }

        private void labelMainColor_DragDrop(object sender, DragEventArgs e)
        {
            if (_monorail == null)
                return;
            _monorail.EntityMonorail.MainColor = (Color)e.Data?.GetData(typeof(Color));
            DrawMonorail();
        }

        private void labelAdditionalColor_DragDrop(object sender, DragEventArgs e)
        {
            if (_monorail == null || _monorail is not DrawingExtendedMonorail)
                return;
            ((EntityExtendedMonorail)_monorail.EntityMonorail).AdditionalColor = (Color)e.Data?.GetData(typeof(Color));
            DrawMonorail();
        }

        /// <summary>
        /// Добавление монорельса
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonAdd_Click(object sender, EventArgs e)
        {
            EventAddMonorail?.Invoke(_monorail);
            Close();
        }
    }
}
